import json
import xml.etree.cElementTree as ET

# https://developer.work.weixin.qq.com/document/path/99399#%E5%8A%A0%E5%AF%86%E4%B8%8E%E5%9B%9E%E5%A4%8D

class RspMsg(object):
    def __init__(self):
        self.msg_type = None
        self.visible_to_user = None
        self.xml_tree = ET.Element('xml')

    def insert_elem(self, name, value):
        curr_node = self.xml_tree

        for n in name.split('/'):
            if curr_node.find(n) is None:
                e = ET.Element(n)
                curr_node.append(e)
            curr_node = curr_node.find(n)
        curr_node.text = value

    def dump_xml(self):
        self.update_xml()
        return ET.tostring(self.xml_tree, encoding='ascii', method='html')

    def update_xml(self):
        self.insert_elem("MsgType", self.msg_type)
        if self.visible_to_user is not None:
            self.insert_elem("VisibleToUser", "|".join([str(x) for x in self.visible_to_user]))


class RspTextMsg(RspMsg):
    def __init__(self):
        super().__init__()
        self.msg_type = 'text'
        self.content = None
        # 主动推送 webhook 时传给 send_text；同步回调 XML 仍只写 Content
        self.mentioned_list = None
        self.mentioned_mobile_list = None

    def update_xml(self):
        super().update_xml()
        self.insert_elem('Text/Content', self.content)


class RspMarkdownMsg(RspMsg):
    def __init__(self):
        super().__init__()
        self.msg_type = 'markdown'
        self.content = None

    def update_xml(self):
        super().update_xml()
        self.insert_elem('Markdown/Content', self.content)


class RspFileMsg(RspMsg):
    """文件类：主动推送用 file_path（先上传再发）；若仅同步回调需自行填写 media_id。"""

    def __init__(self):
        super().__init__()
        self.msg_type = 'file'
        self.file_path = None
        self.media_id = None

    def update_xml(self):
        super().update_xml()
        mid = self.media_id if self.media_id is not None else ''
        self.insert_elem('File/MediaId', mid)


class RspImageMsg(RspMsg):
    """图片类，字段与 webhook send image 一致。"""

    def __init__(self):
        super().__init__()
        self.msg_type = 'image'
        self.base64_image_data = None
        self.md5 = None

    def update_xml(self):
        super().update_xml()
        self.insert_elem('Image/Base64', self.base64_image_data if self.base64_image_data is not None else '')
        self.insert_elem('Image/Md5', self.md5 if self.md5 is not None else '')


class RspNewsMsg(RspMsg):
    """图文类，单条 article，与 demo 中 send_news 一致。"""

    def __init__(self):
        super().__init__()
        self.msg_type = 'news'
        self.title = None
        self.description = None
        self.url = None
        self.pic_url = None

    def update_xml(self):
        super().update_xml()
        self.insert_elem('News/Articles/Item/Title', self.title if self.title is not None else '')
        self.insert_elem('News/Articles/Item/Description', self.description if self.description is not None else '')
        self.insert_elem('News/Articles/Item/Url', self.url if self.url is not None else '')
        self.insert_elem('News/Articles/Item/PicUrl', self.pic_url if self.pic_url is not None else '')


def _optional_json_list(val):
    """表单里的 mentioned_list 等：支持 JSON 数组字符串或已是 list。"""
    if val is None or val == "":
        return None
    if isinstance(val, (list, tuple)):
        return list(val)
    if isinstance(val, str):
        s = val.strip()
        if not s:
            return None
        try:
            x = json.loads(s)
            if isinstance(x, list):
                return x
        except (json.JSONDecodeError, TypeError):
            pass
    return None


def rsp_msg_from_active_params(p):
    """
    将主动发送接口（如 /active_send）的表单参数转为 Rsp* 消息，与 handler 返回类型一致。
    p 需实现 .get(key)；未知 msg_type 或缺少 msg_type 时返回 None。
    """
    if p is None:
        return None
    mt = p.get("msg_type")
    if not mt:
        return None
    if mt == "text":
        r = RspTextMsg()
        r.content = p.get("content")
        r.mentioned_list = _optional_json_list(p.get("mentioned_list"))
        r.mentioned_mobile_list = _optional_json_list(p.get("mentioned_mobile_list"))
        return r
    if mt == "markdown":
        r = RspMarkdownMsg()
        r.content = p.get("content")
        return r
    if mt == "file":
        r = RspFileMsg()
        r.file_path = p.get("file_path")
        r.media_id = p.get("media_id")
        return r
    if mt == "image":
        r = RspImageMsg()
        r.base64_image_data = p.get("base64_image_data")
        r.md5 = p.get("md5")
        return r
    if mt == "news":
        r = RspNewsMsg()
        r.title = p.get("title")
        r.description = p.get("description")
        r.url = p.get("url")
        r.pic_url = p.get("pic_url")
        return r
    return None





from .app import WecomBotServer
from .rsp_msg import (
    RspFileMsg,
    RspImageMsg,
    RspMarkdownMsg,
    RspMsg,
    RspNewsMsg,
    RspTextMsg,
    rsp_msg_from_active_params,
)
from .req_msg import ReqMsg

__author__ = "Pan Zhongxian(panzhongxian0532@gmail.com)"
__license__ = "MIT"

__all__ = [
    "WecomBotServer",
    "RspMsg",
    "ReqMsg",
    "RspTextMsg",
    "RspMarkdownMsg",
    "RspFileMsg",
    "RspImageMsg",
    "RspNewsMsg",
    "rsp_msg_from_active_params",
]



from .req_msg import ReqMsg
import xml.etree.cElementTree as ET

msg_str_list = ["""<xml>
	<WebhookUrl> <![CDATA[https://qyapi.weixin.qq.com/xxxxxxx]]></WebhookUrl>
	<ChatId><![CDATA[wrkSFfCgAALFgnrSsWU38puiv4yvExuw]]></ChatId>
	<PostId><![CDATA[bpkSFfCgAAWeiHos2p6lJbG3_F2xxxxx]]></PostId>
	<ChatType>single</ChatType>
	<GetChatInfoUrl><![CDATA[https://qyapi.weixin.qq.com/cgi-bin/webhook/get_chat_info?code=m49c5aRCdEP8_QQdZmTNR52yJ5TLGcIMzaLJk3x5KqY]]></GetChatInfoUrl>
	<From>
		<UserId>zhangsan</UserId>
		<Name><![CDATA[张三]]></Name>
		<Alias><![CDATA[jackzhang]]></Alias>
	</From>
	<MsgType>text</MsgType>
	<Text>
		<Content><![CDATA[@RobotA hello robot]]></Content>
	</Text>
	<MsgId>abcdabcdabcd</MsgId>
</xml>""",
                """<xml>
                    <WebhookUrl> <![CDATA[https://qyapi.weixin.qq.com/xxxxxxx]]></WebhookUrl>
                    <ChatId><![CDATA[wrkSFfCgAALFgnrSsWU38puiv4yvExuw]]></ChatId>
                    <ChatType>single</ChatType>
                    <From>
                        <UserId>zhangsan</UserId>
                        <Name><![CDATA[张三]]></Name>
                        <Alias><![CDATA[jackzhang]]></Alias>
                    </From>
                    <MsgType>image</MsgType>
                    <Image>
                        <ImageUrl><![CDATA[https://p.qpic.cn/pic_wework/2085796/353325df3d4200754264e0521fd8f01e32a878aae7e52a/0]]></ImageUrl>
                    </Image>
                    <MsgId>abcdabcdabcd</MsgId>
                </xml>""",

                """<xml>
                    <WebhookUrl> <![CDATA[https://qyapi.weixin.qq.com/xxxxxxx]]></WebhookUrl>
                    <ChatId><![CDATA[wrkSFfCgAALFgnrSsWU38puiv4yvExuw]]></ChatId>
                    <ChatType>single</ChatType>
                    <GetChatInfoUrl><![CDATA[https://qyapi.weixin.qq.com/cgi-bin/webhook/get_chat_info?code=m49c5aRCdEP8_QQdZmTNR52yJ5TLGcIMzaLJk3x5KqY]]></GetChatInfoUrl>
                    <From>
                        <UserId>zhangsan</UserId>
                        <Name><![CDATA[张三]]></Name>
                        <Alias><![CDATA[jackzhang]]></Alias>
                    </From>
                    <MsgType>event</MsgType>
                    <Event>
                        <EventType><![CDATA[add_to_chat]]></EventType>
                    </Event>
                    <AppVersion><![CDATA[2.8.12.1551]]></AppVersion>
                    <MsgId>abcdabcdabcd</MsgId>
                </xml>""",
                """<xml>
                   <WebhookUrl><![CDATA[http://in.qyapi.weixin.qq.com/cgi-bin/webhook/send?key=xxxxx]]></WebhookUrl>
                   <ChatId><![CDATA[xxxxx]]></ChatId>
                   <PostId><![CDATA[bpkSFfCgAAWeiHos2p6lJbG3_F2xxxxx]]></PostId>
                   <ChatType>single</ChatType>
                   <From>
                       <UserId><![CDATA[zhangsan]]></UserId>
                       <Name><![CDATA[张三]]></Name>
                       <Alias><![CDATA[zhangsan]]></Alias>
                   </From>
                   <MsgId><![CDATA[xxxxx]]></MsgId>
                   <MsgType><![CDATA[attachment]]></MsgType>
                   <Attachment>
                       <CallbackId><![CDATA[btn_for_show_more]]></CallbackId>
                       <Actions>
                           <Name><![CDATA[btn_more]]></Name>
                           <Value><![CDATA[btn_more]]></Value>
                           <Type><![CDATA[button]]></Type>
                       </Actions>
                   </Attachment>
               </xml>""",
                """<xml>
                   <WebhookUrl><![CDATA[https://qyapi.weixin.qq.com/xxxxxxxx]]></WebhookUrl>
                   <ChatId><![CDATA[wrkSFfCgAAskBzQgmxxxxxxxxxxxx]]></ChatId>
                   <MsgId><![CDATA[CAEQi5HR8wUY/NGagIOAgAMgXQ==]]></MsgId>
                   <ChatType><![CDATA[group]]></ChatType>
                   <From>
                       <UserId><![CDATA[T434200000]]></UserId>
                       <Name><![CDATA[张三]]></Name>
                       <Alias><![CDATA[jackzhang]]></Alias>
                   </From>
                   <MsgType><![CDATA[mixed]]></MsgType>
                   <MixedMessage>
                       <MsgItem>
                           <MsgType><![CDATA[text]]></MsgType>
                           <Text>
                               <Content><![CDATA[@机器人 这是今日的测试情况]]></Content>
                           </Text>
                       </MsgItem>
                       <MsgItem>
                           <MsgType><![CDATA[image]]></MsgType>
                           <Image>
                               <ImageUrl><![CDATA[http://p.qpic.cn/pic_wework/2698515288/54528347764eac2d194c2ce90d83769c62e478f59e706815/0]]></ImageUrl>
                           </Image>
                       </MsgItem>
                   </MixedMessage>
               </xml>"""]

for msg_str in msg_str_list:
    xml_tree = ET.fromstring(msg_str)
    msg = ReqMsg.create_msg(xml_tree)
    print(msg.msg_type)
    print(msg.from_user)
    print(msg.chat_type)
    print(msg.chat_id)
    print(msg.webhook_url)
    print(msg.msg_id)
    if msg.msg_type == 'text':
        print(msg.content)
    elif msg.msg_type == 'event':
        print(msg.event_type)
    elif msg.msg_type == 'image':
        print(msg.image_url)
    elif msg.msg_type == 'attachment':
        print(msg.callback_id)
        for action in msg.actions:
            print(action.name)
            print(action.value)
            print(action.type)
    elif msg.msg_type == 'mixed':
        for msg_item in msg.msg_items:
            print(msg_item.msg_type)
            if msg_item.msg_type == 'text':
                print(msg_item.content)
            elif msg_item.msg_type == 'image':
                print(msg_item.image_url)
    else:
        print("unknown msg type")
    print()



# https://developer.work.weixin.qq.com/document/path/99399#%E8%A7%A3%E5%AF%86%E5%90%8E%E7%9A%84%E6%B6%88%E6%81%AF%E7%BB%93%E6%9E%84%E4%BD%93
class UserInfo(object):
    def __init__(self, en_name, cn_name, user_id):
        self.en_name = en_name
        self.cn_name = cn_name
        self.user_id = user_id

    def __str__(self):
        return f"en_name: {self.en_name}, cn_name: {self.cn_name}, user_id: {self.user_id}"


class ReqMsg(object):
    def __init__(self, xml_tree):
        user = xml_tree.find('From')
        self.from_user = UserInfo(user.find('Alias').text, user.find('Name').text, user.find('UserId').text)
        self.msg_type = xml_tree.find('MsgType').text
        self.chat_type = xml_tree.find('ChatType').text
        self.chat_id = xml_tree.find('ChatId').text
        self.webhook_url = xml_tree.find('WebhookUrl').text
        self.msg_id = xml_tree.find('MsgId').text
        # GetChatInfoUrl

    @staticmethod
    def create_msg(xml_tree):
        msg_type = xml_tree.find('MsgType').text
        if msg_type == 'text':
            return TextReqMsg(xml_tree)
        elif msg_type == 'event':
            return EventReqMsg(xml_tree)
        elif msg_type == 'image':
            return ImageReqMsg(xml_tree)
        elif msg_type == 'attachment':
            return AttachmentReqMsg(xml_tree)
        elif msg_type == 'mixed':
            return MixedMessageReqMsg(xml_tree)
        else:
            return None


class TextReqMsg(ReqMsg):
    def __init__(self, xml_tree):
        super().__init__(xml_tree)
        self.msg_type = 'text'
        self.content = xml_tree.find('Text').find('Content').text


class EventReqMsg(ReqMsg):
    def __init__(self, xml_tree):
        super().__init__(xml_tree)
        self.msg_type = 'event'
        self.event_type = xml_tree.find('Event').find('EventType').text
        # self.event_key = None


class ImageReqMsg(ReqMsg):
    def __init__(self, xml_tree):
        super().__init__(xml_tree)
        self.msg_type = 'image'
        self.image_url = xml_tree.find('Image').find('ImageUrl').text


class AttachmentAction(object):
    def __init__(self, name, value, type_):
        self.name = name
        self.value = value
        self.type = type_


class AttachmentReqMsg(ReqMsg):
    def __init__(self, xml_tree):
        super().__init__(xml_tree)
        self.msg_type = 'attachment'
        self.callback_id = xml_tree.find('Attachment').find('CallbackId').text
        self.actions = []
        e = xml_tree.find('Attachment').find('Actions')
        self.actions.append(AttachmentAction(e.find('Name').text, e.find('Value').text, e.find('Type').text))


class SimpleTextMsg(object):
    def __init__(self, xml_tree):
        self.msg_type = 'text'
        self.content = xml_tree.find('Text').find('Content').text


class SimpleImageMsg(object):
    def __init__(self, xml_tree):
        self.msg_type = 'image'
        self.image_url = xml_tree.find('Image').find('ImageUrl').text


class MixedMessageReqMsg(ReqMsg):
    def __init__(self, xml_tree):
        super().__init__(xml_tree)
        self.msg_type = 'mixed'
        self.msg_items = []
        for e in xml_tree.find('MixedMessage'):
            if e.find('MsgType').text == 'text':
                self.msg_items.append(SimpleTextMsg(e))
            elif e.find('MsgType').text == 'image':
                self.msg_items.append(SimpleImageMsg(e))
            else:
                raise Exception("unknown msg type")



import inspect
import logging
import os
import queue
import threading
import xml.etree.cElementTree as ET

import requests
from flask import Flask, request
from wx_crypt import WXBizMsgCrypt, WxChannel_Wecom

from .req_msg import ReqMsg
from .rsp_msg import RspTextMsg, rsp_msg_from_active_params


# 参考文档：https://km.woa.com/articles/show/387107?kmref=search&from_page=1&no=2#10128


def _encode_rsp(wx_cpt, rsp_str):
    xml = ET.Element('xml')
    ET.SubElement(xml, 'MsgType').text = 'markdown'
    markdown = ET.SubElement(xml, 'Markdown')
    ET.SubElement(markdown, 'Content').text = rsp_str
    plain = ET.tostring(xml).decode()

    # 加密消息
    params = request.args
    timestamp = params.get("timestamp")
    nonce = params.get("nonce")
    ret, rsp = wx_cpt.EncryptMsg(plain, nonce, timestamp)
    if ret != 0:
        print("err: encrypt fail: " + str(ret))
    return rsp


class WecomBotServer(object):
    """_message_handler 在后台线程执行；同步回包最多等待 FIRST_HANDLER_RSP_TIMEOUT_SEC 秒（含「等 handler 返回」与「生成器首条 yield」）。超时则空回包，结果改为主动推送。注意 handler 内勿依赖 Flask 的 request 线程局部变量。"""
    FIRST_HANDLER_RSP_TIMEOUT_SEC = 4

    def __init__(self, name, host, port, path, token=None, aes_key=None, corp_id=None, bot_key=None,
                 active_msg_path="/active_send"):
        """
        :param name:
        :param host:
        :param port:
        :param path:
        :param token:
        :param aes_key:
        :param corp_id:
        :param bot_key:
        :param active_msg_path: 主动发送消息的路径
        """
        self.host = host
        self.port = port
        self.path = path
        self.active_msg_path = active_msg_path
        self._bot_key = bot_key if bot_key is not None else os.getenv("WX_BOT_KEY")
        self._token = token if token is not None else os.getenv("WX_BOT_TOKEN")
        self._aes_key = aes_key if aes_key is not None else os.getenv("WX_BOT_AES_KEY")
        self._corp_id = corp_id if corp_id is not None else os.getenv("WX_BOT_CORP_ID", default="")
        self._app = Flask(name)
        self._message_handler = None
        self._event_handler = None
        self._error_handler = None
        self.name = name
        self.logger = logging.getLogger()

    def set_message_handler(self, handler):
        self._message_handler = handler

    def set_event_handler(self, handler):
        self._event_handler = handler

    def set_error_handler(self, handler):
        self._error_handler = handler

    def set_flask_error_handler(self, handler):
        self._app.errorhandler(Exception)(handler)

    def run(self):
        if self._message_handler is None:
            raise Exception("message handler is not set")
        if self._event_handler is None:
            raise Exception("event handler is not set")
        self._app.get(self.path)(self.handle_bot_call_get)
        self._app.post(self.path)(self.handle_bot_call_post)
        self._app.post(self.active_msg_path)(self.handle_active_send)
        self._app.run(host=self.host, port=self.port)

    def handle_active_send(self):
        # 避免外网直接访问：判断来源IP如果非本地地址，直接返回
        if request.remote_addr != "127.0.0.1":
            return "Invalid request"

        p = request.values
        rsp = rsp_msg_from_active_params(p)
        if rsp is None:
            return "Invalid msg_type"
        send_ret = self.send_active_message(p.get("chat_id"), rsp)
        return "发送消息结果：" + send_ret

    def send_active_message(self, chat_id, rsp_msg):
        """按 Rsp* 主动发送，与 handler 返回类型一致。"""
        if not chat_id:
            return "缺少 chat_id"
        if rsp_msg is None:
            return "缺少消息"
        mt = rsp_msg.msg_type
        if mt == "file":
            if rsp_msg.file_path:
                return self.send_file(chat_id, rsp_msg.file_path)
            if rsp_msg.media_id:
                return self.proactively_send(chat_id, "file", "文件", {"file": {"media_id": rsp_msg.media_id}})
            return "文件消息缺少 file_path 或 media_id"
        if mt == "markdown":
            body = rsp_msg.content if rsp_msg.content is not None else ""
            return self.send_markdown(chat_id, body)
        if mt == "text":
            text = rsp_msg.content if rsp_msg.content is not None else ""
            return self.send_text(
                chat_id,
                text,
                mentioned_list=rsp_msg.mentioned_list,
                mentioned_mobile_list=rsp_msg.mentioned_mobile_list,
            )
        if mt == "image":
            return self.send_encoded_image(
                chat_id,
                rsp_msg.base64_image_data,
                rsp_msg.md5,
            )
        if mt == "news":
            return self.send_news(
                chat_id,
                rsp_msg.title,
                rsp_msg.description,
                rsp_msg.url,
                rsp_msg.pic_url,
            )
        return "Invalid msg_type"

    def _empty_sync_rsp(self):
        r = RspTextMsg()
        r.content = ""
        return r

    def _dispatch_message_handler_with_timeout(self, msg):
        """在后台执行 _message_handler，首包（返回体或生成器首条）超过 FIRST_HANDLER_RSP_TIMEOUT_SEC 则空回包并改主动推送。"""
        chat_id = msg.chat_id
        out = queue.Queue(maxsize=1)

        def worker():
            try:
                if len(inspect.signature(self._message_handler).parameters) == 2:
                    raw = self._message_handler(msg, self)
                else:
                    raw = self._message_handler(msg)
                out.put((True, raw))
            except Exception:
                self.logger.exception("message handler failed")
                out.put((False, None))

        threading.Thread(target=worker, daemon=True).start()

        try:
            ok, raw = out.get(timeout=self.FIRST_HANDLER_RSP_TIMEOUT_SEC)
        except queue.Empty:

            def on_late():
                ok_late, raw_late = out.get()
                if not ok_late:
                    return
                if inspect.isgenerator(raw_late):
                    try:
                        for item in raw_late:
                            if item is None:
                                continue
                            send_ret = self.send_active_message(chat_id, item)
                            if send_ret == "Invalid msg_type":
                                self.logger.warning(
                                    "send_active_message: unsupported msg_type %s",
                                    item.msg_type)
                    except Exception:
                        self.logger.exception("message handler generator failed")
                else:
                    self.send_active_message(chat_id, raw_late)

            threading.Thread(target=on_late, daemon=True).start()
            return self._empty_sync_rsp()

        if not ok:
            return self._empty_sync_rsp()

        if inspect.isgenerator(raw):
            return self._handle_message_handler_generator(msg, raw)
        return raw

    def get_crypto_obj(self):
        return WXBizMsgCrypt(self._token, self._aes_key, self._corp_id, channel=WxChannel_Wecom)

    def handle_bot_call_get(self):
        # 获取请求参数
        params = request.args
        msg_signature = params.get("msg_signature")
        timestamp = params.get("timestamp")
        nonce = params.get("nonce")
        encrypted_echo_str = params.get("echostr")
        wx_cpt = self.get_crypto_obj()
        ret, decrypted_echo_str = wx_cpt.VerifyURL(msg_signature, timestamp, nonce, encrypted_echo_str)
        if ret != 0:
            return None
        return decrypted_echo_str

    def handle_bot_call_post(self):
        # 获取请求参数
        params = request.args
        msg_signature = params.get("msg_signature")
        timestamp = params.get("timestamp")
        nonce = params.get("nonce")
        wx_cpt = self.get_crypto_obj()

        # 解密出明文的echostr
        ret, msg = wx_cpt.DecryptMsg(request.data, msg_signature, timestamp, nonce)
        self.logger.info(f"decrypted msg: {msg.decode()}")
        if ret != 0:
            if self._error_handler:
                self._error_handler(ret)
            else:
                return None
            # 获取所有的查询参数

        # 解密后的数据是xml格式，用python的标准库xml.etree.cElementTree可以解析
        xml_tree = ET.fromstring(msg)
        msg = ReqMsg.create_msg(xml_tree)
        if msg.msg_type == 'event':
            rsp_msg = self._event_handler(msg)
        else:  # 消息
            if msg.msg_type == 'text' and msg.chat_type == 'group':
                msg.content = msg.content.replace(f"@{self.name}", "")
            rsp_msg = self._dispatch_message_handler_with_timeout(msg)

        nonce = params.get("nonce")
        ret, rsp = wx_cpt.EncryptMsg(rsp_msg.dump_xml(), nonce, timestamp)
        if ret != 0:
            print("err: encrypt fail: " + str(ret))
        return rsp

    def _handle_message_handler_generator(self, msg, gen):
        chat_id = msg.chat_id
        q = queue.Queue()
        done = object()

        def producer():
            try:
                for rsp in gen:
                    q.put(rsp)
            except Exception:
                self.logger.exception("message handler generator failed")
            finally:
                q.put(done)

        def drain():
            while True:
                item = q.get()
                if item is done:
                    break
                if item is not None:
                    send_ret = self.send_active_message(chat_id, item)
                    if send_ret == "Invalid msg_type":
                        self.logger.warning(
                            "send_active_message: unsupported msg_type %s", item.msg_type)

        threading.Thread(target=producer, daemon=True).start()

        try:
            first = q.get(timeout=self.FIRST_HANDLER_RSP_TIMEOUT_SEC)
        except queue.Empty:
            first = None

        if first is None:
            threading.Thread(target=drain, daemon=True).start()
            return self._empty_sync_rsp()
        if first is done:
            return self._empty_sync_rsp()

        threading.Thread(target=drain, daemon=True).start()
        return first

    def upload_file(self, file_path):
        filename = os.path.basename(file_path)
        try:
            # 打开文件并上传
            with open(file_path, 'rb') as file:
                files = {'file': (filename, file)}
                response = requests.post(
                    url=f'https://qyapi.weixin.qq.com/cgi-bin/webhook/upload_media?key={self._bot_key}&type=file',
                    files=files)
                # 检查响应
                if response.status_code != 200 and response.json().get("errcode") == 0:
                    return None
                return response.json()['media_id']
        except:
            return None

    def proactively_send(self, chat_id, msg_type, msg_type_name, msg_data):
        """"""
        try:
            payload = {
                "chatid": chat_id,
                "msgtype": msg_type,
            }
            payload.update(msg_data)

            r = requests.post(url=f'https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={self._bot_key}',
                              json=payload)
            if r.status_code == 200 and r.json().get("errcode") == 0:
                return f"发送{msg_type_name}成功"
            else:
                return f"发送{msg_type_name}失败"
        except:
            return f"发送{msg_type_name}失败"

    def send_file(self, chat_id, file_path):
        media_id = self.upload_file(file_path)
        if media_id is None:
            return "上传文件失败"

        return self.proactively_send(chat_id, "file", "文件", {"file": {"media_id": media_id}})

    def send_markdown(self, chat_id, content):
        return self.proactively_send(chat_id, "markdown", "Markdown", {"markdown": {"content": content}})

    def send_text(self, chat_id, content, mentioned_list=None, mentioned_mobile_list=None):
        msg_data = {
            "text": {
                "content": content,
            }
        }
        if mentioned_list is not None:
            msg_data["text"]["mentioned_list"] = mentioned_list
        if mentioned_mobile_list is not None:
            msg_data["text"]["mentioned_mobile_list"] = mentioned_mobile_list
        return self.proactively_send(chat_id, "text", "文本", msg_data)

    def send_encoded_image(self, chat_id, base64_image_data, md5):
        return self.proactively_send(chat_id, "image", "图片", {"image": {"base64": base64_image_data, "md5": md5}})

    def send_news(self, chat_id, title, description, url, pic_url):
        return self.proactively_send(chat_id, "news", "图文", {"news": {"articles": [
            {
                "title": title,
                "description": description,
                "url": url,
                "picurl": pic_url
            }
        ]}})
